<?php

/**
 * @file
 * Set of unified functions to process comments for all versions of Drupal.
 *
 */
 
 /**
 * Find a number of recent comments. This is done in two steps.
 *   1. Find the n (specified by $number) nodes that have the most recent
 *      comments.  This is done by querying node_comment_statistics which has
 *      an index on last_comment_timestamp, and is thus a fast query.
 *   2. Loading the information from the comments table based on the nids found
 *      in step 1.
 *
 * @param $number (optional) The maximum number of comments to find.
 * @return $comments An array of comment objects each containing a nid,
 *   subject, cid, and timstamp, or an empty array if there are no recent
 *   comments visible to the current user.
 */

function com2vb_get_recent($number = 10) {
  // Select the $number nodes (visible to the current user) with the most
  // recent comments. This is efficient due to the index on
  // last_comment_timestamp.

  $result = db_query_range("SELECT nc.nid FROM {node_comment_statistics} nc, {vb_thread} vb, " ._vb_prefix() ."thread t WHERE nc.comment_count > 0 and vb.dr_nid = nc.nid and t.threadid = vb.vb_thread ORDER BY t.lastpost", 0, 0, $number);

  $nids = array();
  while ($row = db_fetch_object($result)) {
    $nids[] = $row->nid;
  }

  $comments = array();
  if (!empty($nids)) {
    // From among the comments on the nodes selected in the first query,
    // find the $number most recent comments.
	$result = db_query('SELECT vb_thread FROM {vb_thread} WHERE dr_nid in ('. implode(',', $nids) .')');

	$threads = array();
	while ($row = db_fetch_object($result)) {
	  $threads[] = $row->vb_thread;
	}
    $result = db_query_range('SELECT vb.dr_nid as nid, p.title as subject, p.postid as cid, p.dateline as timestamp FROM ' ._vb_prefix() .'post p INNER JOIN {vb_thread} vb ON vb.vb_thread = p.threadid WHERE p.threadid IN ('. implode(',', $threads) .') AND p.visible = %d ORDER BY p.postid DESC', !COMMENT_PUBLISHED, 0, $number);
    while ($comment = db_fetch_object($result)) {
      $comments[] = $comment;
    }
  }
  return $comments;
}

/**
 * Implementation of hook_perm().
 */
function com2vb_perm() {
  return array('access comments', 'post comments', 'administer comments', 'post comments without approval', 'manage users creation in forum');
}

/**
 * Implementation of hook_link().
 */
function com2vb_link($type, $node = NULL, $teaser = FALSE) {
  $links = array();

  if ($type == 'node' && $node->comment) {

    if ($teaser) {
      // Main page: display the number of comments that have been posted.

      if (user_access('access comments')) {
        $all = com2vb_num_all($node->nid);

        if ($all) {
          $links['comment_comments'] = array(
            'title' => format_plural($all, '1 comment', '@count comments'),
            'href' => "node/$node->nid",
            'attributes' => array('title' => t('Jump to the first comment of this posting.')),
            'fragment' => 'comments'
          );

          $new = com2vb_num_new($node->nid);

          if ($new) {
            $links['comment_new_comments'] = array(
              'title' => format_plural($new, '1 new comment', '@count new comments'),
              'href' => "node/$node->nid",
              'attributes' => array('title' => t('Jump to the first new comment of this posting.')),
              'fragment' => 'new'
            );
          }
        }
        else {
          if ($node->comment == COMMENT_NODE_READ_WRITE) {
            if (user_access('post comments')) {
              $links['comment_add'] = array(
                'title' => t('Add new comment'),
                'href' => "comment/reply/$node->nid",
                'attributes' => array('title' => t('Add a new comment to this page.')),
                'fragment' => 'comment-form'
              );
            }
            else {
              $links['comment_forbidden']['title'] = theme('com2vb_post_forbidden', $node->nid);
            }
          }
        }
      }
    }
    else {
      // Node page: add a "post comment" link if the user is allowed to
      // post comments, if this node is not read-only, and if the comment form isn't already shown

      if ($node->comment == COMMENT_NODE_READ_WRITE) {
        if (user_access('post comments')) {
          if (variable_get('comment_form_location', COMMENT_FORM_SEPARATE_PAGE) == COMMENT_FORM_SEPARATE_PAGE) {
            $links['comment_add'] = array(
              'title' => t('Add new comment'),
              'href' => "comment/reply/$node->nid",
              'attributes' => array('title' => t('Share your thoughts and opinions related to this posting.')),
              'fragment' => 'comment-form'
            );
          }
        }
        else {
          $links['comment_forbidden']['title'] = theme('com2vb_post_forbidden', $node->nid);
        }
      }
    }
  }

  if ($type == 'comment') {
    $links = com2vb_links($node, $teaser);
  }
  if (isset($links['comment_forbidden'])) {
    $links['comment_forbidden']['html'] = TRUE;
  }

  return $links;
}

/**
 * Implementation of hook_nodeapi().
 *
 */
function com2vb_nodeapi(&$node, $op, $arg = 0) {
  switch ($op) {
    case 'load':
      return db_fetch_array(db_query("SELECT t.lastpost as last_comment_timestamp, t.lastposter as last_comment_name, t.replycount as comment_count FROM " ._vb_prefix() ."thread t, {vb_thread} dr WHERE dr.dr_nid = %d and t.threadid = dr.vb_thread", $node->nid));
      break;

    case 'prepare':
      if (!isset($node->comment)) {
        $node->comment = variable_get("comment_$node->type", COMMENT_NODE_READ_WRITE);
      }
      break;

    case 'insert':
      db_query('INSERT INTO {node_comment_statistics} (nid, last_comment_timestamp, last_comment_name, last_comment_uid, comment_count) VALUES (%d, %d, NULL, %d, 0)', $node->nid, $node->changed, $node->uid);
	  _vb_start_forum($node, 0);
	  _vb_start_post($node);
      break;
	  
	case 'update':
	  $forum_id = db_result(db_query('SELECT forum_id FROM {vb_thread} WHERE dr_nid = %d', $node->nid));
	  _vb_start_forum($node, $forum_id);
	  break;
	  
	case 'delete':
		$node->vb_thread = db_result(db_query('SELECT vb_thread FROM {vb_thread} WHERE dr_nid = %d', $node->nid));
		$node->cid = $node->vb_thread;
		$op = _com2vb_forum_on_action();
		db_query('DELETE FROM {vb_thread} WHERE dr_nid = %d', $node->nid);
		_com2vb_delete_thread($node, $op);
		break;

    case 'update index':
      $text = '';
      $comments = db_query('SELECT p.title as subject, p.pagetext as comment, 1 as format FROM ' ._vb_prefix() .'post p, {vb_thread} vb WHERE vb.dr_nid = %d AND p.threadid = vb.vb_thread AND p.visible = %d', $node->nid, COMMENT_PUBLISHED);

      while ($comment = db_fetch_object($comments)) {
        $text .= '<h2>'. check_plain($comment->subject) .'</h2>'. check_markup($comment->comment, $comment->format, FALSE);
      }
      return $text;

    case 'search result':
      $comments = db_result(db_query('SELECT t.replycount as comment_count FROM ' ._vb_prefix() .'thread t, {vb_thread} vb WHERE vb.dr_nid = %d AND t.threadid = vb.vb_thread', $node->nid));

      return format_plural($comments, '1 comment', '@count comments');

    case 'rss item':
      if ($node->comment != COMMENT_NODE_DISABLED) {
        return array(array('key' => 'comments', 'value' => url('node/'. $node->nid, NULL, 'comments', TRUE)));
      }
      else {
        return array();
      }
  }
}

/**
 * Implementation of hook_user().
 */
function com2vb_user($op, $edit, &$account, $category = NULL) {
  global $user;
  $access = user_access('manage users creation in forum');
  $vbuser = db_result(db_query('SELECT vb_userid FROM {vb_users} WHERE dr_userid = %d', $account->uid));
  
  switch ($op) {
  case 'form':
  case 'register':
    	$form['comment_settings'] = array(
      	'#type' => 'fieldset',
      	'#title' => t('Comment settings'),
      	'#collapsible' => TRUE,
      	'#weight' => 4);
		
		$form['comment_settings']['newvbuser'] = array(
		'#type' => 'checkbox',
		'#default_value' => variable_get('com2vb_new_user_in_forum', COM2VB_NEW_USER_IN_FORUM),
		'#title' => 'Create the same user in forum',
		'#description' => t('Users with <em>manage users creation in forum</em> permission can override this settings'),
		'#disabled' => TRUE,
		);

  	if ($category == 'account' && $account->uid == $user->uid) {
    // when user tries to edit his own data
    	$form['comment_settings']['signature'] = array(
      	'#type' => 'textarea',
      	'#title' => t('Signature'),
      	'#default_value' => $edit['signature'],
      	'#description' => t('Your signature will be publicly displayed at the end of your comments.'));
	}
		
	if (!$vbuser && $access) {
		$form['comment_settings']['newvbuser']['#disabled'] = FALSE;
	}
   	return $form;
  	
  case 'delete':
    db_query('UPDATE {comments} SET uid = 0 WHERE uid = %d', $account->uid);
	db_query('UPDATE {node_comment_statistics} SET last_comment_uid = 0 WHERE last_comment_uid = %d', $account->uid);
//	$vbuser = db_result(db_query('SELECT vb_userid FROM {vb_users} WHERE dr_userid = %d', $user->uid));
	db_query('UPDATE ' ._vb_prefix() .'post SET userid = 0 WHERE userid = %d', $vbuser);
    db_query('UPDATE ' ._vb_prefix() .'thread SET lastpostid = 0 WHERE lastpostid = %d', $vbuser);
	db_query('DELETE FROM {vb_users} WHERE dr_userid = %d AND vb_userid = %d', $account->uid, $vbuser);
  break;
  
  case 'insert':
  	if ($edit['newvbuser']) {
		$vbuserid = _com2vb_user_create($account, $edit);
		if ($vbuserid > 0) {
			$text = t('User was created in forum with Id = ').$vbuserid;
			drupal_set_message($text, 'status');
			watchdog('user', $text, WATCHDOG_NOTICE);
		}
		elseif ($vbuserid == 0) {
			$text = t('Unknown error while creating a user in forum');
			drupal_set_message($text, 'error');
			watchdog('user', $text, WATCHDOG_ERROR);
		}
		elseif ($vbuserid < 0) {
			$text = t('The name %name is used in forum.', array('%name' => $edit['name']));
			drupal_set_message($text, 'error');
			watchdog('user', $text, WATCHDOG_ERROR);
		}
	}
  break;
  
  case 'validate':
  	if (variable_get('com2vb_new_user_check', COM2VB_NEW_USER_CHECK)) {
	_com2vb_user_validate($account, $edit, $vbuser);
	}
  break;
  
  case 'update':
	if (!$vbuser && $edit['newvbuser']) {
		com2vb_user('insert', $edit, $account, $category);
		break;
	}
  	$result = _com2vb_user_update($account, $edit, $vbuser);
	if ($result) {
		$text = t('User %name was updated in forum', array('%name' => $edit['name']));
		drupal_set_message($text, 'status');
		watchdog('user', $text, WATCHDOG_NOTICE);
	}
	else {
		$text = t('User %name was not updated in forum', array('%name' => $edit['name']));
		drupal_set_message($text, 'error');
		watchdog('user', $text, WATCHDOG_ERROR);
	}
  break;
  
  case 'login':
  	if (_com2vb_user_login($account)) {
		drupal_set_message(t('You are successfully loged to forum'), 'status');
		watchdog('user', t('User %user loged to forum.', array('%user' => $account->name)), WATCHDOG_NOTICE);
	}
	else {
		drupal_set_message(t('You are not loged to forum or banned in forum'), 'error');
		watchdog('user', t('Login to forum failed for user %user.', array('%user' => $account->name)), WATCHDOG_ERROR);
	}
  break;
  
  case 'logout':
  	_com2vb_user_logout($account);
  }
}

/**
 * This is *not* a hook_access() implementation. This function is called
 * to determine whether the current user has access to a particular comment.
 *
 * Authenticated users can edit their comments as long they have not been
 * replied to. This prevents people from changing or revising their
 * statements based on the replies to their posts.
 */
function com2vb_access($op, $comment) {
  global $user;

  if ($op == 'edit') {
    return ($user->uid && $user->uid == $comment->uid && com2vb_num_replies($comment->cid) == 0) || user_access('administer comments');
  }
}

/**
 *
 */
function com2vb_node_url() {
  return arg(0) .'/'. arg(1);
}

/**
 * Accepts a submission of new or changed comment content.
 *
 * @param $edit
 *   A comment array.
 *
 * @return
 *   If the comment is successfully saved the comment ID is returned. If the comment
 *   is not saved, FALSE is returned.
 */
function com2vb_save($edit) {

  global $user, $base_url, $node;
  
// converting drupal's user to vBulletin's user
  $edit['uid'] = db_result(db_query("SELECT vb_userid as uid FROM {vb_users} WHERE dr_userid = %d", $edit['uid']));
  //checking mode of posting: real posting or moving of old comments from drupal
  if ($edit['export'] == 0) $edit['ipaddress'] = _com2vb_ip_address();
  
  if (user_access('post comments') && (user_access('administer comments') || node_comment_mode($edit['nid']) == COMMENT_NODE_READ_WRITE)) {
    if (!form_get_errors()) {
      if ($edit['cid']) {
        // Update the comment in the database.
        _com2vb_update_node_statistics($edit['nid']);

        // Allow modules to respond to the updating of a comment.
        com2vb_invoke_comment($edit, 'update');

        // Add an entry to the watchdog log.
        watchdog('content', t('Comment: updated %subject.', array('%subject' => $edit['subject'])), WATCHDOG_NOTICE, l(t('view'), 'node/'. $edit['nid'], array('fragment' => 'comment-'. $edit['cid'])));

      }
      else {
        // Check for duplicate comments. Note that we have to use the
        // validated/filtered data to perform such check.

        $duplicate = db_result(db_query("SELECT COUNT(p.postid) FROM " ._vb_prefix() ."post p, {vb_thread} vb WHERE vb.dr_nid = %d AND p.threadid = vb.vb_thread AND p.parentid = %d AND p.title = '%s' AND p.pagetext = '%s'", $edit['nid'], $edit['pid'], $edit['subject'], $edit['comment']), 0);
        if ($duplicate != 0) {
          watchdog('content', t('Comment: duplicate %subject.', array('%subject' => $edit['subject'])), WATCHDOG_WARNING);
        }

        // Add the comment to database.
        $edit['status'] = user_access('post comments without approval') ? COMMENT_PUBLISHED : COMMENT_NOT_PUBLISHED;
        $roles = variable_get('comment_roles', array());
        $score = 0;

        foreach (array_intersect(array_keys($roles), array_keys($user->roles)) as $rid) {
          $score = max($roles[$rid], $score);
        }

        // Here we are building the thread field. See the documentation for
        // comment_render().
        if ($edit['pid'] == 0) {
          // This is a comment with no parent comment (depth 0): we start
          // by retrieving the maximum thread level.
          //finding id of start post in forum
		  $edit['pid'] = db_result(db_query('SELECT t.firstpostid FROM {vb_thread} vb, ' ._vb_prefix() .'thread t WHERE vb.dr_nid = %d AND t.threadid = vb.vb_thread', $edit['nid']));
          //if start post is absence, than we create it.
		  if ($edit['pid'] == 0) {
		  	_com2vb_save($edit, 1);
		  }
        }
		if (variable_get('com2vb_html_conv', COM2VB_HTML_CONV)) {
		  $edit['comment'] = html2bbcode($edit['comment']);
		  }
		  
        $edit['vb_thread'] = db_result(db_query("SELECT vb_thread FROM {vb_thread} WHERE dr_nid = %d", $edit['nid']));
		$edit['pid'] = db_result(db_query('SELECT t.firstpostid FROM {vb_thread} vb, ' ._vb_prefix() .'thread t WHERE vb.dr_nid = %d AND t.threadid = vb.vb_thread', $edit['nid']));
        _com2vb_save($edit, 0);
		// increasing post counter of vBulettin's user
		if ($edit['uid'] > 0) db_query("UPDATE " ._vb_prefix() ."user SET posts = posts+1 WHERE userid = %d", $edit['uid']);
		// founding id of newest comment and updating comments statistic in vBulletin
        $edit['cid'] = db_result(db_query("SELECT MAX(LAST_INSERT_ID(postid)) FROM " ._vb_prefix() ."post"));
		// Increasing reply's counter of thread
		db_query("UPDATE " ._vb_prefix() ."thread SET replycount = replycount+1 WHERE threadid = %d", $edit['vb_thread'], $edit['timestamp']);
		// Updating information about last post in thread
		db_query("UPDATE " ._vb_prefix() ."thread SET lastpostid = %d, lastpost = %d, lastposter = '%s', dateline = %d WHERE threadid = %d AND lastpost < %d", $edit['cid'], $edit['timestamp'], $edit['name'], $edit['timestamp'], $edit['vb_thread'], $edit['timestamp']);
		// updating stastic of board in vBulletin
		$thread_req = db_fetch_object(db_query("SELECT forumid, title FROM " ._vb_prefix() ."thread WHERE threadid = %d", $edit['vb_thread']));
		$edit['vb_forumid'] = $thread_req->forumid;
		$edit['vb_threadname'] = $thread_req->title;
		// increasing count of replays
		db_query("UPDATE " ._vb_prefix() ."forum SET replycount = replycount+1 WHERE forumid = %d", $edit['vb_forumid']);
		// updating time of last post if need
		db_query("UPDATE " ._vb_prefix() ."forum SET lastpost = %d, lastposter = '%s', lastpostid = %d, lastthread = '%s', lastthreadid = %d WHERE lastpost < %d AND forumid = %d", $edit['timestamp'], $edit['name'], $edit['cid'], $edit['vb_threadname'], $edit['vb_thread'], $edit['timestamp'], $edit['vb_forumid']);

// copying comment in "native" table {comments} for backward compatibility with other modules, which working with table {comments} directly (Notify, Akismet, etc.)
        db_query("INSERT INTO {comments} (cid, nid, pid, uid, subject, format, hostname, timestamp, status, thread, name, mail, homepage) VALUES (%d, %d, %d, %d, '%s', %d, '%s', %d, %d, '%s', '%s', '%s', '%s')", $edit['cid'], $edit['nid'], $edit['pid'], $edit['uid'], $edit['subject'], $edit['format'], $edit['ipaddress'], $edit['timestamp'], $edit['status'], $thread, $edit['name'], $edit['mail'], $edit['homepage']);

        _com2vb_update_node_statistics($edit['nid']);

        // Tell the other modules a new comment has been submitted.
        com2vb_invoke_comment($edit, 'insert');

        // Add an entry to the watchdog log.
        watchdog('content', t('Comment: added %subject.', array('%subject' => $edit['subject'])), WATCHDOG_NOTICE, l(t('view'), 'node/'. $edit['nid'], array('fragment' => 'comment-'. $edit['cid'])));
      }

      // Clear the cache so an anonymous user can see his comment being added.
      cache_clear_all();

      // Explain the approval queue if necessary, and then
      // redirect the user to the node he's commenting on.
      if ($edit['status'] == COMMENT_NOT_PUBLISHED) {
        drupal_set_message(t('Your comment has been queued for moderation by site administrators and will be published after approval.'));
      }
      return $edit['cid'];
    }
    else {
      return FALSE;
    }
  }
  else {
    $txt = t('Comment: unauthorized comment submitted or comment submitted to a closed node %subject.', array('%subject' => $edit['subject']));
    watchdog('content', $txt, WATCHDOG_WARNING);
    drupal_set_message($txt, 'error');
    return FALSE;
  }
}

function com2vb_links($comment, $return = 1) {
  global $user;

  $links = array();

  // If we are viewing just this comment, we link back to the node.
  if ($return) {
    $links['comment_parent'] = array(
      'title' => t('parent'),
      'href' => com2vb_node_url(),
      'fragment' => "comment-$comment->cid"
    );
  }

  if (node_comment_mode($comment->nid) == COMMENT_NODE_READ_WRITE) {
    if (user_access('administer comments') && user_access('post comments')) {
/*
      $links['comment_delete'] = array(
        'title' => t('delete'),
        'href' => "comment/delete/$comment->cid"
      );

      $links['comment_edit'] = array(
        'title' => t('edit'),
        'href' => "comment/edit/$comment->cid"
      );
*/
      $links['comment_reply'] = array(
        'title' => t('reply'),
        'href' => "comment/reply/$comment->nid/$comment->cid"
      );
	  
      $links['comment_host'] = array(
        'title' => t('Host: ' . $comment->hostname),
      );

      $links['comment_mail'] = array(
        'title' => t('E-mail: ' . $comment->mail),
      );

    }
    else if (user_access('post comments')) {
      if (com2vb_access('edit', $comment)) {
        $links['comment_edit'] = array(
          'title' => t('edit'),
          'href' => "comment/edit/$comment->cid"
        );
      }
      $links['comment_reply'] = array(
        'title' => t('reply'),
        'href' => "comment/reply/$comment->nid/$comment->cid"
      );
    }
    else {
      $links['comment_forbidden']['title'] = theme('com2vb_post_forbidden', $comment->nid);
    }
  }

  return $links;
}

/**
 * Menu callback; delete a comment.
 */
function com2vb_delete($cid = NULL) {
  $comment = db_fetch_object(db_query('SELECT c.*, u.name AS registered_name, u.uid FROM {comments} c INNER JOIN {users} u ON u.uid = c.uid WHERE c.cid = %d', $cid));
  $comment->name = $comment->uid ? $comment->registered_name : $comment->name;

  $output = '';

  if (is_object($comment) && is_numeric($comment->cid)) {
    $output = drupal_get_form('com2vb_confirm_delete', $comment);
  }
  else {
    drupal_set_message(t('The comment no longer exists.'));
  }

  return $output;
}

/**
 * Stub for backward compatibility with Akismet module
 */
function comment_operations($action = NULL) {
	return com2vb_operations($action);
}
 
/**
 * Comment operations. We offer different update operations depending on
 * which comment administration page we're on.
 */
function com2vb_operations($action = NULL) {
  if ($action == 'publish') {
    $operations = array(
      'publish' => array(t('Publish the selected comments'), 'UPDATE ' ._vb_prefix() .'post SET visible = 1 WHERE postid = %d')
    );
  }
  else if ($action == 'unpublish') {
    $operations = array(
      'unpublish' => array(t('Unpublish the selected comments'), 'UPDATE ' ._vb_prefix() .'post SET visible = 0 WHERE postid = %d')
    );
  }
  else {
    $operations = array(
      'publish' => array(t('Publish the selected comments'), 'UPDATE ' ._vb_prefix() .'post SET visible = 1 WHERE postid = %d'),
      'unpublish' => array(t('Unpublish the selected comments'), 'UPDATE ' ._vb_prefix() .'post SET visible = 0 WHERE postid = %d')
    );
  }
  return $operations;
}

/**
*** misc functions: helpers, privates, history
**/

/**
 * Load the entire comment by cid.
 */
function _com2vb_load($cid) {
  return db_fetch_object(db_query('SELECT * FROM ' ._vb_prefix() .'post WHERE postid = %d', $cid));
}

/*
 *
 */
function com2vb_num_all($nid) {
  static $cache;

  if (!isset($cache[$nid])) {
    $cache[$nid] = db_result(db_query('SELECT t.replycount AS comment_count FROM {vb_thread} vb, ' ._vb_prefix() .'thread t WHERE vb.dr_nid = %d AND t.threadid = vb.vb_thread', $nid));
  }
  return $cache[$nid];
}

/*
 *
 */
function com2vb_num_replies($pid) {
  static $cache;

  if (!isset($cache[$pid])) {
    $cache[$pid] = db_result(db_query('SELECT COUNT(postid) FROM ' ._vb_prefix() .'post WHERE parentid = %d AND visible = %d', $pid, !COMMENT_PUBLISHED));
  }

  return $cache[$pid];
}

/**
 * get number of new comments for current user and specified node
 *
 * @param $nid node-id to count comments for
 * @param $timestamp time to count from (defaults to time of last user access
 *   to node)
 */
function com2vb_num_new($nid, $timestamp = 0) {
  global $user;

  if ($user->uid) {
    // Retrieve the timestamp at which the current user last viewed the
    // specified node.
    if (!$timestamp) {
      $timestamp = node_last_viewed($nid);
    }
    $timestamp = ($timestamp > NODE_NEW_LIMIT ? $timestamp : NODE_NEW_LIMIT);

    // Use the timestamp to retrieve the number of new comments.
    $result = db_result(db_query('SELECT COUNT(p.postid) FROM {node} n LEFT JOIN {vb_thread} vbt ON vbt.dr_nid = n.nid LEFT JOIN ' ._vb_prefix() .'post p ON p.threadid = vbt.vb_thread WHERE n.nid = %d AND p.dateline > %d AND p.visible = %d', $nid, $timestamp, !COMMENT_PUBLISHED));

    return $result;
  }
  else {
    return 0;
  }

}

function com2vb_validate($edit) {
  global $user;

  // Invoke other validation handlers
  com2vb_invoke_comment($edit, 'validate');

  if (isset($edit['date'])) {
    // As of PHP 5.1.0, strtotime returns FALSE upon failure instead of -1.
    if (strtotime($edit['date']) <= 0) {
      form_set_error('date', t('You have to specify a valid date.'));
    }
  }
  if (isset($edit['author']) && !$account = user_load(array('name' => $edit['author']))) {
    form_set_error('author', t('You have to specify a valid author.'));
  }

  // Check validity of name, mail and homepage (if given)
  if (!$user->uid || isset($edit['is_anonymous'])) {
    if (variable_get('comment_anonymous', COMMENT_ANONYMOUS_MAYNOT_CONTACT) > COMMENT_ANONYMOUS_MAYNOT_CONTACT) {
      if ($edit['name']) {
        $taken = db_result(db_query("SELECT COUNT(uid) FROM {users} WHERE LOWER(name) = '%s'", $edit['name']), 0);

        if ($taken != 0) {
          form_set_error('name', t('The name you used belongs to a registered user.'));
        }

      }
      else if (variable_get('comment_anonymous', COMMENT_ANONYMOUS_MAYNOT_CONTACT) == COMMENT_ANONYMOUS_MUST_CONTACT) {
        form_set_error('name', t('You have to leave your name.'));
      }

      if ($edit['mail']) {
        if (!valid_email_address($edit['mail'])) {
          form_set_error('mail', t('The e-mail address you specified is not valid.'));
        }
      }
      else if (variable_get('comment_anonymous', COMMENT_ANONYMOUS_MAYNOT_CONTACT) == COMMENT_ANONYMOUS_MUST_CONTACT) {
        form_set_error('mail', t('You have to leave an e-mail address.'));
      }

      if ($edit['homepage']) {
        if (!valid_url($edit['homepage'], TRUE)) {
          form_set_error('homepage', t('The URL of your homepage is not valid. Remember that it must be fully qualified, i.e. of the form <code>http://example.com/directory</code>.'));
        }
      }
    }
  }

  return $edit;
}

/**
 * Stub for backward compatibility with Akismet module
 */
function _comment_delete_thread($comment, $op = 'nothing') {
	_com2vb_delete_thread($comment, $op);
}

/**
 * Processing of deleting a comment or thread. Action depends on 'com2vb_forum_on_action' variable
 */
function _com2vb_delete_thread($comment, $op = 'nothing') {
  if (!is_object($comment) || !is_numeric($comment->cid)) {
    watchdog('content', t('Can not delete non-existent comment.'), WATCHDOG_WARNING);
    return;
  }
  
  if ($comment->vb_thread != '') {
  	$op .= '_vb';
	}
	
	//watchdog('com2vb', t('_com2vb_delete_thread: $op = %op', array('%op' => $op)));
	//watchdog('com2vb', t('_com2vb_delete_thread: $comment->vb_thread = %op', array('%op' => $comment->vb_thread)));
  
	switch($op) {
		case 'nothing':
			// Delete the comment from {comments} and link to a forum's thread:
			db_query('DELETE FROM {comments} WHERE cid = %d', $comment->cid);
			//db_query('DELETE FROM ' ._vb_prefix() .'post WHERE postid = %d', $comment->cid);
			watchdog('content', t('Comment: removed from Drupal %subject.', array('%subject' => $comment->subject)));
			break;
		
		case 'unpublish':
			db_query('UPDATE ' ._vb_prefix() .'post SET visible = 0 WHERE postid = %d', $comment->cid);
			break;
			
		case 'delete':
			// Delete the comment from {comments} and link to a forum's thread:
			db_query('DELETE FROM {comments} WHERE cid = %d', $comment->cid);
			db_query('DELETE FROM ' ._vb_prefix() .'post WHERE postid = %d', $comment->cid);
			db_query('UPDATE ' ._vb_prefix() .'user SET posts = posts-1 WHERE userid = %d', $comment->userid);
			watchdog('content', t('Comment: deleted %subject.', array('%subject' => $comment->subject)));
			break;
		
		case 'nothing_vb':
			$vb = db_query('SELECT firstpostid FROM ' ._vb_prefix() .'thread WHERE threadid = %d', $comment->vb_thread);
			$comment = db_fetch_object(db_query('SELECT postid as cid FROM ' ._vb_prefix() .'post WHERE parentid = %d', $vb));
			$op = 'nothing';
			return;
		
		case 'unpublish_vb':
			db_query('UPDATE ' ._vb_prefix() .'thread SET visible = 0 WHERE threadid = %d', $comment->vb_thread);
			return;
			//break;
		
		case 'close_vb':
			db_query('UPDATE ' ._vb_prefix() .'thread SET open = 0 WHERE threadid = %d', $comment->vb_thread);
			return;
			//break;
		
		case 'delete_vb':
			$vb = db_result(db_query('SELECT firstpostid FROM ' ._vb_prefix() .'thread WHERE threadid = %d', $comment->vb_thread));
			db_query('DELETE FROM ' ._vb_prefix() .'thread WHERE threadid = %d', $comment->vb_thread);
			//db_query('DELETE FROM ' ._vb_prefix() .'post WHERE postid = %d', $vb);
			//$comment = db_fetch_object(db_query('SELECT postid as cid FROM ' ._vb_prefix() .'post WHERE parentid = %d', $vb));
			$op = 'delete';
			//return;
		}

	com2vb_invoke_comment($comment, 'delete');

	// Delete the comment's replies
	$result = db_query('SELECT p.postid as cid, p.threadid, p.parentid, p.userid, p.title as subject, p.dateline, u.name AS registered_name, u.uid FROM ' ._vb_prefix() .'post p LEFT JOIN {vb_users} vbu ON vbu.vb_userid = p.userid LEFT JOIN {users} u ON u.uid = vbu.dr_userid WHERE p.threadid = %d', $comment->vb_thread);
	while ($comment = db_fetch_object($result)) {
    $comment->name = $comment->uid ? $comment->registered_name : $comment->name;
    _com2vb_delete_thread($comment, $op);
  }
}

/**
 * Return an array of viewing modes for comment listings.
 *
 * We can't use a global variable array because the locale system
 * is not initialized yet when the comment module is loaded.
 */
function _com2vb_get_modes() {
  return array(
    COMMENT_MODE_FLAT_COLLAPSED => t('Flat list - collapsed'),
    COMMENT_MODE_FLAT_EXPANDED => t('Flat list - expanded'),
    COMMENT_MODE_THREADED_COLLAPSED => t('Threaded list - collapsed')
  );
}

/**
 * Return an array of viewing orders for comment listings.
 *
 * We can't use a global variable array because the locale system
 * is not initialized yet when the comment module is loaded.
 */
function _com2vb_get_orders() {
  return array(
    COMMENT_ORDER_NEWEST_FIRST => t('Date - newest first'),
    COMMENT_ORDER_OLDEST_FIRST => t('Date - oldest first')
  );
}

/**
 * Return an array of "comments per page" settings from which the user
 * can choose.
 */
function _com2vb_per_page() {
  return drupal_map_assoc(array(1, 2, 3, 5, 10, 20));
}

/**
 * Return a current comment display setting
 *
 * $setting can be one of these: 'mode', 'sort', 'comments_per_page'
 */
function _com2vb_get_display_setting($setting, $node = NULL) {
  global $user;
  
  if (!isset($node)) {
  	$node->type = '';
  }
  else {
  	$node->type = '_'. $node->type;
	}

  if (isset($_GET[$setting])) {
    $value = $_GET[$setting];
  }
  else {
    // get the setting's site default
    switch ($setting) {
      case 'mode':
        $default = variable_get('comment_default_mode'. $node->type, COMMENT_MODE_FLAT_EXPANDED);
        break;
      case 'sort':
        $default = variable_get('comment_default_order'. $node->type, COMMENT_ORDER_NEWEST_FIRST);
        break;
      case 'comments_per_page':
        $default = variable_get('comment_default_per_page'. $node->type, '5');
    }
    if (variable_get('comment_controls'. $node->type, COMMENT_CONTROLS_HIDDEN) == COMMENT_CONTROLS_HIDDEN) {
      // if comment controls are disabled use site default
      $value = $default;
    }
    else {
      // otherwise use the user's setting if set
      if ($user->$setting) {
        $value = $user->$setting;
      }
      else if ($_SESSION['comment_'. $setting]) {
        $value = $_SESSION['comment_'. $setting];
      }
      else {
        $value = $default;
      }
    }
  }
  return $value;
}

/**
 * Updates the comment statistics for a given node. This should be called any
 * time a comment is added, deleted, or updated.
 *
 * The following fields are contained in the node_comment_statistics table.
 * - last_comment_timestamp: the timestamp of the last comment for this node or the node create stamp if no comments exist for the node.
 * - last_comment_name: the name of the anonymous poster for the last comment
 * - last_comment_uid: the uid of the poster for the last comment for this node or the node authors uid if no comments exists for the node.
 * - comment_count: the total number of approved/published comments on this node.
 */
function _com2vb_update_node_statistics($nid) {
  $count = db_result(db_query('SELECT COUNT(p.postid) as cid FROM ' ._vb_prefix() .'post p, {vb_thread} vb WHERE vb.dr_nid = %d AND p.threadid = vb.vb_thread AND p.visible = %d', $nid, !COMMENT_PUBLISHED));

  // comments exist
  if ($count > 0) {
    $last_reply = db_fetch_object(db_query_range('SELECT p.postid AS cid, p.username AS name, p.dateline AS timestamp, u.dr_userid AS uid FROM ' ._vb_prefix() .'post p, {vb_users} u, {vb_thread} vb  WHERE vb.dr_nid = %d AND p.threadid = vb.vb_thread AND p.visible = %d ORDER BY p.postid DESC', $nid, !COMMENT_PUBLISHED, 0, 1));
    db_query("UPDATE {node_comment_statistics} SET comment_count = %d, last_comment_timestamp = %d, last_comment_name = '%s', last_comment_uid = %d WHERE nid = %d", $count, $last_reply->timestamp, $last_reply->uid ? '' : $last_reply->name, $last_reply->uid, $nid);
  }

  // no comments
  else {
    $node = db_fetch_object(db_query("SELECT uid, created FROM {node} WHERE nid = %d", $nid));
    db_query("UPDATE {node_comment_statistics} SET comment_count = 0, last_comment_timestamp = %d, last_comment_name = '', last_comment_uid = %d WHERE nid = %d", $node->created, $node->uid, $nid);
  }
}

/**
 * Stub for backward compatibility with Akismet module
 */
function _comment_update_node_statistics($nid) {
	_com2vb_update_node_statistics($nid);
}

/**
 * Stub for backward compatibility with Akismet module
 */
function comment_invoke_comment(&$comment, $op) {
	switch($op) {
		case 'delete':
		db_query('DELETE FROM' ._vb_prefix() .'post WHERE postid = %d', $comment->cid);
		break;
		
		case 'unpublish':
		db_query('UPDATE ' ._vb_prefix() .'post SET visible = 0 WHERE postid = %d', $comment->cid);
		break;
		
		case 'publish':
		db_query('UPDATE ' ._vb_prefix() .'post SET visible = 1 WHERE postid = %d', $comment->cid);
	}
	
	watchdog('com2vb', 'Akismet sad "'.$op.'" comment '.$comment->cid, WATCHDOG_WARNING);
}

/**
 * Invoke a hook_comment() operation in all modules.
 *
 * @param &$comment
 *   A comment object.
 * @param $op
 *   A string containing the name of the comment operation.
 * @return
 *   The returned value of the invoked hooks.
 */
function com2vb_invoke_comment(&$comment, $op) {
  $return = array();
  foreach (module_implements('comment') as $name) {
    $function = $name .'_comment';
    $result = $function($comment, $op);
    if (isset($result) && is_array($result)) {
      $return = array_merge($return, $result);
    }
    else if (isset($result)) {
      $return[] = $result;
    }
  }
  return $return;
}

/**
 * Get a list of forums in vBulletin database
 * @return
 * Associated array containing forums list
 */
function _vb_forum_list() {
	$forum_list_ext = array();
	$categories = array();
	
	$query = db_query('SELECT forumid FROM ' ._vb_prefix() .'forum WHERE NOT options & 4');
	while ($values = db_fetch_object($query)) {
		$categories[$values->forumid] = 'category';
		}
	$forum_list_ext = array_merge(_vb_forum_sublist($categories, $forum_list_ext, -1), $forum_list_ext);

	return $forum_list_ext;
}

/**
 * Helper function for function _vb_forum_list()
 */
function _vb_forum_sublist($categories = array(), &$forum_list_ext = array(), $forumid = -1) {
	$forum_list_int = array();
	$forums = db_query('SELECT forumid, title_clean, displayorder, parentid, parentlist, childlist FROM ' ._vb_prefix() .'forum WHERE parentid = %d ORDER BY displayorder, forumid', $forumid);
	
		while ($values = db_fetch_object($forums)) {
			$values->title_clean = str_repeat('-', substr_count($values->parentlist, ',') - 1). ' '. $values->title_clean;
			$childs = db_result(db_query('SELECT COUNT(forumid) FROM ' ._vb_prefix() .'forum WHERE parentid = %d', $values->forumid));
			if ($categories[$values->forumid] == 'category' && $forumid == -1) {
				if ($childs > 0) $forum_list_int[$values->title_clean] = _vb_forum_sublist($categories, $forum_list_ext, $values->forumid);
				} elseif ($categories[$values->forumid] == 'category') {
					$forum_list_ext[$values->title_clean] = array();
					if ($childs > 0) $forum_list_ext[$values->title_clean] = _vb_forum_sublist($categories, $forum_list_ext, $values->forumid);
					} else {
						$forum_list_int[$values->forumid] = $values->title_clean;
						if ($childs > 0) $forum_list_int = array_merge($forum_list_int, _vb_forum_sublist($categories, $forum_list_ext, $values->forumid));
				}
			}

	return $forum_list_int;
}

/**
 * Get current settings of action on deleting a node
 * @return
 * Id of action
 */
function _com2vb_forum_on_action() {
	return variable_get('com2vb_forum_on_action', COM2VB_FORUM_DEFAULT_ACTION);
}

/**
 * Ancillary function to com2vb_save()
 * If $start = 1, then we are creating a start post
 */
function _com2vb_save($edit, $start = 0) {
	global $base_url;
	
	if ($start) {
		//taking a "preview" of active version of node for inserting in forum
		$dr_teaser = db_fetch_array(db_query('SELECT v.teaser, v.title, n.uid FROM {node_revisions} v, {node} n WHERE n.nid = %d and v.vid = n.vid', $edit['nid']));
		$dr_teaser['teaser'] = node_teaser($dr_teaser['teaser']);
		//converting html tags to bbcode
		if (variable_get('com2vb_html_conv', COM2VB_HTML_CONV)) {
			$dr_teaser['teaser'] = html2bbcode($dr_teaser['teaser']);
		}
		//taking a URL to node
		$dr_teaser['link'] = $base_url.COM2VB_DR_NODE_TEMP.$edit['nid'];
		//formating body of start post in forum
		$fr_body = "[QUOTE]".$dr_teaser['teaser']."[/QUOTE][URL=".$dr_teaser['link']."]".t('Read more')."[/URL]";
		//define the forum to post comment:
		//checking for individual settings for node
		$edit['id_forum'] = db_result(db_query('SELECT forum_id FROM {vb_thread} WHERE dr_nid = %d', $edit['nid']));
		$forum_init = 1;
		//what is default forum to post for this node type?
		if (!$edit['id_forum']) {
			$ntype = 'com2vb_type_' . db_result(db_query('SELECT type FROM {node} WHERE nid = %d', $edit['nid']));
			$edit['id_forum'] = variable_get($ntype, _vb_forum_id());
			$forum_init = 0;
		}
		
		$edit['subject'] =  $dr_teaser['title'];
		$edit['comment'] = $fr_body;
		if ($edit['timestamp'] == 0) $edit['timestamp'] = time();
		}

		// Insert post in forum's database		
		db_query("INSERT INTO " ._vb_prefix() ."post (threadid, parentid, userid, title, pagetext, ipaddress, dateline, visible, username, allowsmilie) VALUES (%d, %d, %d, '%s', '%s', '%s', %d, %d, '%s', 1)", $edit['vb_thread'], $edit['pid'], $edit['uid'], $edit['subject'], $edit['comment'], $edit['ipaddress'], $edit['timestamp'] - 1, 1, $edit['name']);
		
		if ($start) {
			$edit['pid'] = db_result(db_query("SELECT MAX(LAST_INSERT_ID(postid)) FROM " ._vb_prefix() ."post"));
			db_query("INSERT INTO " ._vb_prefix() ."thread (title, firstpostid, lastpostid, lastpost, forumid, open, postusername, postuserid, lastposter, dateline, visible, replycount) VALUES('%s', %d, %d, %d, %d, 1, '%s', %d, '%s', %d, 1, 0)", $dr_teaser['title'], $edit['pid'], $edit['pid'], $edit['timestamp'], $edit['id_forum'], $edit['name'], $edit['uid'], $edit['name'], $edit['timestamp']);
			$edit['vb_thread'] = db_result(db_query("SELECT MAX(LAST_INSERT_ID(threadid)) FROM " ._vb_prefix() ."thread"));
			if ($forum_init == 0) {
				db_query('INSERT INTO {vb_thread} (dr_nid, vb_thread) VALUES (%d, %d)', $edit['nid'], $edit['vb_thread']);
			}
			else {
				db_query('UPDATE {vb_thread} SET vb_thread = %d WHERE dr_nid = %d', $edit['vb_thread'], $edit['nid']);
			}
			db_query("UPDATE " ._vb_prefix() ."post SET threadid = %d WHERE postid = %d", $edit['vb_thread'], $edit['pid']);
			// updating statistic of board in vBulletin
			// increasing count of replays
			db_query("UPDATE " ._vb_prefix() ."forum SET threadcount = threadcount+1 WHERE forumid = %d", $edit['id_forum']);
		}
}

/**
 * Get current settings of forum to post
 * @return
 * Associated array containing form of forums list
 */
function _vb_forum_select($var = NULL, $multiple = FALSE) {

	$def_forum = _vb_forum_id();
	if (isset($var['node_type'])) {
		$def_node_forum = variable_get('com2vb_type_' . $var['node_type'], $def_forum);
		}
		else {
		$def_node_forum = $def_forum;
	}

	return array(
    '#type' => 'select',
	'#title' => t('Default forum to post comments'),
	'#default_value' => $def_node_forum,
	'#options' => _vb_forum_list(),
	'#size' => 5,
	'#maxlength' => 16,
	'#multiple' => $multiple,
  );
}

/**
 * Get a board in vBulletin to post comments
 * @return
 * Id of forum as integer
 */
function _vb_forum_id() {
	return variable_get('com2vb_def_forum', COM2VB_FORUM_POST);
}
	
/**
 * Get a prefix of vBulletin's database
 * @return
 * String containing prefix
 */
function _vb_prefix() {
	return variable_get('com2vb_db_prefix', COM2VB_PREFIX);
}

/**
 * Remove specific to vBulletin extensions of bbcode for correct displaying in Drupal
 */
function _vb_clear_bbcode($text) {
	// [ur="..."] -> [ur=...]
	$text = preg_replace('#(\[url[^"]*?)"([^"]*?)"([^"]*?url\])#i', '$1$2$3', $text);
	// [quote=Anonimous;1364] -> [quote=Anonimous]
	$text = preg_replace('#(\[quote=[^;]*?);([0-9]{1,})(\])#i', '$1$3', $text);
	
	return $text;
 }

function _vb_start_forum($node, $forum_id = 0) {
	if (!$node->com2vb_forum_chk) {
	  	if ($forum_id > 0) :
	  		db_query('UPDATE {vb_thread} SET forum_id = %d WHERE dr_nid = %d', $node->com2vb_forum, $node->nid);
		 	else :
		 		db_query('INSERT INTO {vb_thread} (dr_nid, forum_id) VALUES (%d, %d)', $node->nid, $node->com2vb_forum);
		 	endif;
		}
		elseif ($forum_id > 0) {
			db_query('DELETE FROM {vb_thread} WHERE dr_nid = %d', $node->nid);
		}
}

function _vb_start_post($node) {
	if ($node->status && $node->com2vb_start_post) {
		$edit = array();
		
		$edit['vb_thread'] = db_result(db_query('SELECT vb_thread FROM {vb_thread} WHERE dr_nid = %d', $node->nid));
		$edit['nid'] = $node->nid;
		$edit['uid'] = $node->uid;
		$edit['name'] = db_result(db_query('SELECT name FROM {users} WHERE uid = %d', $edit['uid']));
		//Who will be the author of start post?
		switch (variable_get('com2vb_teaser_author', COM2VB_TEASER_AUTHOR)) {
			case 0:
				$edit['uid'] = db_result(db_query('SELECT vb_userid FROM {vb_users} WHERE dr_userid = %d', $node->uid));
			break;
			
			default:
				$edit['uid'] = 0;
			break;
		}
		_com2vb_save($edit, 1);
		}
}

/*
 * Backport of function ip_address() from Drupal 6
 */
function _com2vb_ip_address() {

  static $ip_address = NULL;

  if (!isset($ip_address)) {
    $ip_address = $_SERVER['REMOTE_ADDR'];
    if (variable_get('reverse_proxy', 0) && array_key_exists('HTTP_X_FORWARDED_FOR', $_SERVER)) {
      // If an array of known reverse proxy IPs is provided, then trust
      // the XFF header if request really comes from one of them.
      $reverse_proxy_addresses = variable_get('reverse_proxy_addresses', array());
      if (!empty($reverse_proxy_addresses) && in_array($ip_address, $reverse_proxy_addresses, TRUE)) {
        // If there are several arguments, we need to check the most
        // recently added one, i.e. the last one.
        $ip_address = array_pop(explode(',', $_SERVER['HTTP_X_FORWARDED_FOR']));
      }
    }
  }

  return $ip_address;

}